import pygame
from pygame.locals import *

import playgame
import display
import status
import resource
import mob
import intro
import terrain
from helper import *


def mainmenu():
	if DEBUG:
		playgame.startgame()
	
	f = resource.get_font('Aquiline', 92)
	img = resource.get_image('TowerPhoto')
	screen = pygame.display.get_surface()
	screen.blit(img, (0,0))
	screen.blit(f.render("Wizard's Solitude", True, pygame.Color('blue')), (40, -40))
	
	menu = MainMenu()

	clock = pygame.time.Clock()
	result = None
	done = False
	
	while not done:
		for event in pygame.event.get():
			if event.type == MOUSEBUTTONDOWN and event.button == 1:
				result = menu.click()
		
		if result == "Play Game":
			result = None
			intro.show_story()
			playgame.startgame()
			pygame.mouse.set_visible(True)
		elif result == "Quit Game":
			result = None
			done = True
		
		menu.draw()
		
		pygame.display.flip()
		clock.tick(60)
	




summon_actions = [mob.Slime(), mob.RockGolem(),
	terrain.BrickWall(), terrain.HedgeWall(),
	terrain.PitTrap(), terrain.ManaTrap(), terrain.FlameTower()]

all_summon_actions = [mob.Slime(), mob.RockGolem(),
	terrain.BrickWall(), terrain.HedgeWall(),
	terrain.PitTrap(), terrain.ManaTrap(), terrain.FlameTower()]

class Result:
	[Quit] = range(1)

class Menu:
	def __init__(self, shade_gray):
		screen = pygame.display.get_surface()
		
		self.stash = pygame.Surface((1024,768), 0, screen)
		self.stash.blit(screen, (0,0))
		self.active = False
		
		gray = pygame.Surface((1024,768), 0, screen)
		gray.fill(pygame.Color(100, 100, 100))
		gray.set_alpha(160)
		
		self.surf = pygame.Surface((1024,768), 0, screen)
		self.surf.blit(self.stash, (0,0))
		if shade_gray:
			self.surf.blit(gray, (0,0))
		
		self.font = resource.get_font('bluehigh', 26)
		self.line = 30
		self.x_pos = 300
		self.width = 424
		self.y_offs = 30
		
		self.title = "Menu"
	
	def draw(self):
		self.active = True
		
		screen = pygame.display.get_surface()
		screen.blit(self.surf, (0,0))

		box = pygame.Rect(self.x_pos - 5, self.y_offs - 5, self.width + 10, (self.line * len(self.actions)) + 10)
		pygame.draw.rect(screen, pygame.Color('black'), box)
		pygame.draw.rect(screen, pygame.Color('white'), box, 1)

		screen.blit(self.font.render(self.title, True, pygame.Color('white')), (self.x_pos + 5, self.y_offs - 25))

		mouse = pygame.mouse.get_pos()
		if not (mouse[0] < self.x_pos or mouse[0] > self.x_pos + self.width):
			highlight = (mouse[1] - self.y_offs) / self.line
			if highlight >= 0 and highlight < len(self.actions):
				pygame.draw.rect(screen, pygame.Color(180, 180, 180), (self.x_pos, (highlight * self.line) + self.y_offs, self.width, self.line))

	def click(self):
		mouse = pygame.mouse.get_pos()
		
		if mouse[0] < self.x_pos or mouse[0] > self.x_pos + self.width:
			return None
		
		row = (mouse[1] - self.y_offs) / self.line
		if row >= 0 and row < len(self.actions):
			return self.actions[row]
		return None
	
	def close(self):
		self.active = False

class MainMenu(Menu):
	def __init__(self):
		Menu.__init__(self, False)
		self.actions = ["Play Game", "Load Game", "Quit Game"]
		self.title = "Menu"
		self.x_pos = 350
		self.width = 324
		self.y_offs = 200
	
	def draw(self):
		Menu.draw(self)
		screen = pygame.display.get_surface()
		y_pos = self.y_offs
		
		for act in self.actions:
			screen.blit(self.font.render(act, True, pygame.Color('white')), (self.x_pos, y_pos+ self.line/4))
			y_pos += self.line
		

class GameMenu(Menu):
	def __init__(self):
		Menu.__init__(self, False)
		self.actions = ["Save Game", "Load Game", "Quit"]
		self.title = "Game Menu"
		self.x_pos = 350
		self.width = 324
		self.y_offs = 200
	
	def draw(self):
		Menu.draw(self)
		screen = pygame.display.get_surface()
		y_pos = self.y_offs
		
		for act in self.actions:
			screen.blit(self.font.render(act, True, pygame.Color('white')), (self.x_pos, y_pos+ self.line/4))
			y_pos += self.line
	
class SummonMenu(Menu):
	def __init__(self):
		Menu.__init__(self, True)
		mana = status.get_mana()
		#self.actions = [act for act in summon_actions if act.cost < mana and act.cost > 0]
		self.actions = summon_actions
		self.title = "Summoning"

	
	def draw(self):
		Menu.draw(self)
		
		screen = pygame.display.get_surface()
		y_pos = self.y_offs
		
		for act in self.actions:
			icon = act.sprite.image
			icon = pygame.transform.scale(icon, (icon.get_width()/2, icon.get_height() / 2))
			screen.blit(self.font.render(str(act.cost) + " Mana", True, pygame.Color('white')), (self.x_pos, y_pos + self.line/4))
			screen.blit(icon, (self.x_pos + 100,y_pos+30-icon.get_height()))
			screen.blit(self.font.render(act.name, True, pygame.Color('blue')), (self.x_pos + 140, y_pos + self.line/4))
			y_pos += self.line		
		
		# restore screen when selection is made
		# check if summoning is allowed at location
		# create whatever was summoned and deduct mana if allowed
		# resume game
	
	def click(self):
		action = Menu.click(self)
		if action == None:
			return
		else:
			#print action.name
			self.active = False
			return action


class QuitConfirm(Menu):
	def __init__(self):
		Menu.__init__(self, True)
		self.actions = ["Quit", "Cancel"]
		self.x_pos = 450
		self.width = 124
		self.y_offs = 200
		self.title = "Quit?"
	
	def draw(self):
		Menu.draw(self)
		screen = pygame.display.get_surface()
		y_pos = self.y_offs
		
		for act in self.actions:
			screen.blit(self.font.render(act, True, pygame.Color('white')), (self.x_pos, y_pos+ self.line/4))
			y_pos += self.line

	def click(self):
		action = Menu.click(self)
		if action == None:
			return
		elif action == "Quit":
			self.active = False
			return Result.Quit
		else:
			self.active = False
			return None








