import customsprites
import display
import terrain
from helper import *


class Mood:
	[Fight, Flee, Seek, Smash, Wander, Wait, Pitted] = range(7)

class Mob:
	def __init__(self):
		#self.sprite = customsprites.MobSprite('DefenderTest')
		#self.sprite.mob = self
		self.mood = Mood.Wait
		self.rage = 0
		self.enemy = False
		self.vitality = 100
		self.damage = 0
		self.attack = 10
		#self.sprite.cycle = 1
		#self.sprite.speed = 4
		self.vision = 5
		self.cost = 1
		self.name = "Generic Mob"
	
	def set_position(self, t_pos):
		t_pos = tup2rect(t_pos)
		self.position = t_pos
		self.sprite.set_location(display.get_screen().tile_to_screen(t_pos))

	def move_to(self, t_pos):
		t_pos = tup2rect(t_pos)
		
		if self.mood != Mood.Flee:
			for m in display.get_map().mobs:
				if m.position == t_pos:
					return False
		
		if t_pos.topleft == display.get_map().t_origin and not (self.enemy and self.mood == Mood.Flee):
			return False
		
		if t_pos.topleft == display.get_map().t_goal and not (self.enemy and self.mood == Mood.Seek):
			return False
		
		# Deal with walking onto terrain
		t = display.get_map().terrain[t_pos.left][t_pos.top]
		if t != None and t.elevated:
			self.sprite.elevation = 20
		else:
			self.sprite.elevation = 0
		
		self.position = t_pos
		self.sprite.move_to(display.get_screen().tile_to_screen(t_pos))
		
		# tell the terrain we're walking on it
		if t != None:
			t.stepped_on(self)
		
		return True

	def update(self, tick):
		if self.sprite.idle:
			self.work_it()
	
	def work_it(self):
		pass

	def smash_terrain(self, t_pos):
		ter = display.get_map().terrain[t_pos.left][t_pos.top]
		self.sprite.show_attack(ter)
		ter.damage += self.attack * 2
	
	def enemy_visible(self):
		for m in display.get_map().mobs:
			if m.enemy != self.enemy and self.dist_to(m) < self.vision and m.damage < m.vitality:
				if (not m.enemy) or (m.mood != Mood.Flee):
					return m
		return None
	
	def dist_to(self, mob):
		return abs(self.position.left - mob.position.left) + abs(self.position.top - mob.position.top)

	def bash(self, enemy):
		self.sprite.show_attack(enemy)
		enemy.damage += self.attack

	def can_reach(self, enemy):
		return self.dist_to(enemy) <= 1

	def get_slow_factor(self):
		return max(1, (display.get_map().weights[self.position[0]][self.position[1]] / 50) + 1)

	def clicked(self):
		pass
	
	def pitted(self):
		self.sprite.pitted()
	
	def banish(self):
		display.get_map().del_mob(self)


class DefenderMob(Mob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('Slime')
		self.sprite.mob = self
		self.sprite.speed = 4
		self.sprite.cycle = 2
		self.enemy = False
		self.mood = Mood.Wander
		self.cost = 10
		self.name = "Test Defender"

	def work_it(self):
		if self.damage > self.vitality:
			self.banish()
		
		m = self.enemy_visible()
		
		if self.mood == Mood.Wander and m != None:
			self.mood = Mood.Fight
			self.rage = 100
		
		if self.mood == Mood.Fight:
			if m == None:
				self.mood = Mood.Wander
				self.rage = 0
			elif self.can_reach(m):
				self.bash(m)
				self.rage = 50
			elif self.rage > 0:
				self.rage -= 10
				dest = display.get_map().move_toward(self.position, m.position)
				if dest != None:
					self.move_to(dest)
				else:
					self.rage = 0
					self.mood = Mood.Wander
			else:
				self.mood = Mood.Wander
				self.rage = 0
		
		if self.mood == Mood.Flee:
			dest = display.get_map().seek_from(self.position)
			self.move_to(dest)

		if self.mood == Mood.Wander:
			dest = display.get_map().rand_from(self.position)
			self.move_to(dest)


class InvaderMob(Mob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('Slime')
		self.sprite.mob = self
		self.sprite.speed = 4
		self.sprite.cycle = 2
		self.enemy = True
		self.mood = Mood.Seek
		self.vitality = 200
		self.attack = 20
		self.vision = 2
		self.cost = -1
		self.name = "Test Invader"
	
	def work_it(self):
		dest = display.get_map().seek_from(self.position)
		self.move_to(dest)
	
	def work_it(self):
		if self.damage >= self.vitality:
			self.mood = Mood.Flee
			self.vitality = 0
			#display.get_map().invaders_left -= 1
			self.damage = -1
			self.sprite.speed = 6
			self.sprite.cycle = 2
		
		if self.position.topleft == display.get_map().t_goal and self.mood == Mood.Seek:
			display.get_map().you_fail_it()
		elif self.position.topleft == display.get_map().t_origin and self.mood == Mood.Flee:
			display.get_map().del_mob(self)
			
		
		m = self.enemy_visible()
		
		if self.mood != Mood.Flee and m != None:
			self.mood = Mood.Fight
			self.rage = 100
		
		if self.mood == Mood.Fight:
			if m == None:
				self.mood = Mood.Seek
				self.rage = 0
			elif self.can_reach(m):
				self.bash(m)
				self.rage = 50
			elif self.rage > 0:
				self.rage -= 10
				dest = display.get_map().move_toward(self.position, m.position)
				if dest != None:
					self.move_to(dest)
				else:
					self.mood = Mood.Seek
					self.rage = 0
			else:
				self.mood = Mood.Seek
				self.rage = 0
		
		if self.mood == Mood.Seek:
			dest = display.get_map().seek_from(self.position)
			ter = display.get_map().terrain[dest.left][dest.top]
			if display.get_map().seek[dest.left][dest.top] > 200 and ter != None and ter.type == terrain.TType.Wall:
				self.smash_terrain(dest)
			else:
				self.move_to(dest)
		
		if self.mood == Mood.Flee:
			dest = display.get_map().escape_from(self.position)
			self.move_to(dest)

class Brat(InvaderMob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('Brat')
		self.sprite.mob = self
		self.sprite.speed = 4
		self.sprite.cycle = 2
		self.enemy = True
		self.mood = Mood.Seek
		self.vitality = 100
		self.attack = 20
		self.vision = 3
		self.cost = -1
		self.name = "Peasant Brat"

class Dog(InvaderMob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('Dog')
		self.sprite.mob = self
		self.sprite.speed = 6
		self.sprite.cycle = 2
		self.enemy = True
		self.mood = Mood.Seek
		self.vitality = 150
		self.attack = 30
		self.vision = 5
		self.cost = -1
		self.name = "Peasant Mutt"

	def pitted(self):
		pass

class Girl(InvaderMob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('Girl')
		self.sprite.mob = self
		self.sprite.speed = 4
		self.sprite.cycle = 4
		self.enemy = True
		self.mood = Mood.Seek
		self.vitality = 80
		self.attack = 30
		self.vision = 7
		self.cost = -1
		self.name = "Peasant Girl"
	
	def can_reach(self, enemy):
		return self.dist_to(enemy) <= 5

class Slime(DefenderMob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('Slime')
		self.sprite.mob = self
		self.sprite.speed = 4
		self.sprite.cycle = 3
		self.vitality = 80
		self.damage = 0
		self.attack = 15
		self.enemy = False
		self.mood = Mood.Wander
		self.cost = 15
		self.name = "Attack Jelly"


class RockGolem(DefenderMob):
	def __init__(self):
		Mob.__init__(self)
		self.sprite = customsprites.MobSprite('RockGolem')
		self.sprite.mob = self
		self.sprite.speed = 2
		self.sprite.cycle = 4
		self.vitality = 200
		self.damage = 0
		self.attack = 40
		self.enemy = False
		self.mood = Mood.Wander
		self.cost = 50
		self.name = "Rock Golem"
