import pygame
from pygame.locals import *

import map
import mob
import terrain
import resource
import display
import customsprites
import menu
import status
from helper import *

def startgame():	
	pygame.mouse.set_visible(False)
	
	gameloop()

def gameloop():
	end_loop = False
	clock = pygame.time.Clock()
	
	level_count = 0
	#level_maps = [map.FirstMap, map.SecondMap, map.ThirdMap, map.FourthMap, map.FifthMap, map.SixthMap]
	level_maps = [map.FirstMap, map.SecondMap, map.ThirdMap, map.FourthMap, map.SixthMap]
	
	display.get_screen().reset()
	display.get_screen().set_map(level_maps[level_count]())
	display.get_screen().map.render()
	
	cur_map = display.get_screen().map
	
	status.set_mana(200)
	
	gametick = 0
	current_menu = None
	
	summon_type = None
	
	while not end_loop:
		
		gametick = (gametick + 1) % 10080
		
		for event in pygame.event.get():
			if event.type == KEYDOWN:					
				if event.key == Key.quit:
					if current_menu != None:
						current_menu.close()
						current_menu = None
					else:
						current_menu = menu.QuitConfirm()
				elif event.key == pygame.K_s:
					temp = pygame.display.get_surface().copy()
					pygame.image.save(temp, 'scr.PNG')
				elif event.key == Key.menu and current_menu == None:
					current_menu = menu.SummonMenu()
			elif event.type == KEYUP:
				pass
			elif event.type == MOUSEMOTION:
				display.get_screen().cursor_moved()
			elif event.type == MOUSEBUTTONDOWN and event.button == 3:
				thing = display.get_screen().object_clicked()
				if thing != None:
					thing.clicked()
			elif event.type == MOUSEBUTTONDOWN and event.button == 1 and current_menu != None:
				result = current_menu.click()
				if isinstance(current_menu, menu.SummonMenu):
					summon_type = result
				if not current_menu.active:
					current_menu.close()
					current_menu = None
				if result == menu.Result.Quit:
					end_loop = True
			elif event.type == MOUSEBUTTONDOWN and event.button == 1 and current_menu == None and summon_type != None:
				if summon_type.cost > status.get_mana():
					pass
					#print "Insufficient mana!"
				elif cur_map.can_summon_at(display.get_screen().mouse_tile()):
					status.alter_mana(-summon_type.cost)
					cur_map.summon(summon_type.__class__(), display.get_screen().mouse_tile())
				else:
					pass
					#print "Obstruction!"
		
		if cur_map.lost:
			display.get_screen().reset()
			display.get_screen().set_map(cur_map.__class__())
			display.get_screen().map.render()
			cur_map = display.get_screen().map
			cur_map.story_shown = True
			cur_map.calculate_pathing()
			status.set_mana(200)
		elif cur_map.won:
			level_count += 1
			if level_count >= len(level_maps):
				end_loop = True
			else:
				display.get_screen().reset()
				display.get_screen().set_map(level_maps[level_count]())
				display.get_screen().map.render()
				cur_map = display.get_screen().map
				status.set_mana(200)
		
		if current_menu == None:
			if gametick % 80 == 0:
				cur_map.heal_mobs()
			if gametick % 10 == 0:
				status.alter_mana(2)
		
			cur_map.update(gametick)
			display.get_screen().update_status_screen(summon_type)

			display.get_screen().update(gametick)
			
			cur_map.check_story()
		else:
			current_menu.draw()
			display.get_screen().mouse_only()
		
		
		clock.tick(30)
	
	
	if cur_map.won and level_count >= len(level_maps):
		victory()


def victory():
	# show victory image
	f = resource.get_font('Aquiline', 80)
	screen = pygame.display.get_surface()
	text = f.render("The End", True, pygame.Color('white'))
	
	screen.fill(pygame.Color('black'))
	screen.blit(text, ((1024 - text.get_width())/2, 200))
	
	pygame.display.flip()
	
	pygame.time.wait(2000)
	clock = pygame.time.Clock()
	waiting = True
	while waiting:
		for event in pygame.event.get():
			if event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
				waiting = False
			elif event.type == MOUSEMOTION:
				display.get_screen().cursor_moved()
		
		pygame.display.flip()
		clock.tick(10)
